@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2017 Intel Corporation
+    Copyright (c) 2005-2019 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -12,10 +12,6 @@
     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     See the License for the specific language governing permissions and
     limitations under the License.
-
-
-
-
 */
 
 #ifndef _TBB_tbb_statistics_H
@@ -208,7 +204,7 @@ struct statistics_counters {
 
     statistics_counters() { reset(); }
 
-    void reset () { memset( (void*) this, 0, sizeof(statistics_counters) ); }
+    void reset () { memset( static_cast<void*>(this), 0, sizeof(statistics_counters) ); }
 
     counter_type& field ( size_t index ) { return reinterpret_cast<counter_type*>(this)[index]; }
 
