@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // Used in tests that work with TBB scheduler but do not link to the TBB library.
@@ -24,10 +20,17 @@
 #ifndef harness_inject_scheduler_H
 #define harness_inject_scheduler_H
 
+#if HARNESS_DEFINE_PRIVATE_PUBLIC
+#include <string> // merely prevents LNK2019 error to happen (on ICL+VC9 configurations)
+#include <algorithm> // include it first to avoid error on define below
+#define private public
+#define protected public
+#endif
+
 // Suppress usage of #pragma comment
 #define __TBB_NO_IMPLICIT_LINKAGE 1
 
-// Enable preview features if any 
+// Enable preview features if any
 #define __TBB_BUILD 1
 
 #undef DO_ITT_NOTIFY
@@ -71,4 +74,9 @@ int TestMain () {
 #define TestMain TestMainSkipped
 #endif
 
+#if HARNESS_DEFINE_PRIVATE_PUBLIC
+#undef protected
+#undef private
+#endif
+
 #endif /* harness_inject_scheduler_H */
