@@ -1,27 +1,27 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness.h"
+
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
+#endif
+
 #include "tbb/flow_graph.h"
 #include "tbb/task.h"
-
 #include "tbb/atomic.h"
 
 const int N = 1000;
@@ -43,10 +43,11 @@ template< typename T >
 class counting_array_receiver : public tbb::flow::receiver<T> {
 
     tbb::atomic<size_t> my_counters[N];
+    tbb::flow::graph& my_graph;
 
 public:
 
-    counting_array_receiver() {
+    counting_array_receiver(tbb::flow::graph& g) : my_graph(g) {
         for (int i = 0; i < N; ++i )
            my_counters[i] = 0;
     }
@@ -56,20 +57,27 @@ public:
         return v;
     }
 
-    /* override */ tbb::task * try_put_task( const T &v ) {
+    tbb::task * try_put_task( const T &v ) __TBB_override {
         ++my_counters[(int)v];
-        return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
+        return const_cast<tbb::task *>(tbb::flow::internal::SUCCESSFULLY_ENQUEUED);
     }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-    /*override*/void internal_add_built_predecessor(tbb::flow::sender<T> &) {}
-    /*override*/void internal_delete_built_predecessor(tbb::flow::sender<T> &) {}
-    /*override*/void copy_predecessors(std::vector<tbb::flow::sender<T>*> &) {}
-    /*override*/size_t predecessor_count() { return 0; }
-    /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) { }
-#else
-    /*override*/void reset_receiver() { }
+    tbb::flow::graph& graph_reference() __TBB_override {
+        return my_graph;
+    }
+
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
+    built_predecessors_type mbp;
+    built_predecessors_type &built_predecessors() __TBB_override { return mbp; }
+    typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_type predecessor_type;
+    void internal_add_built_predecessor(predecessor_type &) __TBB_override {}
+    void internal_delete_built_predecessor(predecessor_type &) __TBB_override {}
+    void copy_predecessors(predecessor_list_type &) __TBB_override {}
+    size_t predecessor_count() __TBB_override { return 0; }
 #endif
+    void reset_receiver(tbb::flow::reset_flags /*f*/) __TBB_override { }
 
 };
 
@@ -80,22 +88,22 @@ void test_serial_broadcasts() {
     tbb::flow::broadcast_node<T> b(g);
 
     for ( int num_receivers = 1; num_receivers < R; ++num_receivers ) {
-        counting_array_receiver<T> *receivers = new counting_array_receiver<T>[num_receivers];
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        std::vector< counting_array_receiver<T> > receivers(num_receivers, counting_array_receiver<T>(g));
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
         ASSERT(b.successor_count() == 0, NULL);
         ASSERT(b.predecessor_count() == 0, NULL);
-        typename tbb::flow::broadcast_node<T>::successor_vector_type my_succs;
+        typename tbb::flow::broadcast_node<T>::successor_list_type my_succs;
         b.copy_successors(my_succs);
         ASSERT(my_succs.size() == 0, NULL);
-        typename tbb::flow::broadcast_node<T>::predecessor_vector_type my_preds;
+        typename tbb::flow::broadcast_node<T>::predecessor_list_type my_preds;
         b.copy_predecessors(my_preds);
         ASSERT(my_preds.size() == 0, NULL);
 #endif
 
         for ( int r = 0; r < num_receivers; ++r ) {
             tbb::flow::make_edge( b, receivers[r] );
-        } 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        }
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
         ASSERT( b.successor_count() == (size_t)num_receivers, NULL);
 #endif
 
@@ -108,13 +116,10 @@ void test_serial_broadcasts() {
                 ASSERT( receivers[r][n] == 1, NULL );
             }
             tbb::flow::remove_edge( b, receivers[r] );
-        } 
+        }
         ASSERT( b.try_put( (T)0 ), NULL );
-        for ( int r = 0; r < num_receivers; ++r ) 
-            ASSERT( receivers[0][0] == 1, NULL ) ;
-
-        delete [] receivers;
-
+        for ( int r = 0; r < num_receivers; ++r )
+            ASSERT( receivers[0][0] == 1, NULL );
     }
 
 }
@@ -126,24 +131,24 @@ class native_body : private NoAssign {
 
 public:
 
-    native_body( tbb::flow::broadcast_node<T> &b ) : my_b(b) {} 
+    native_body( tbb::flow::broadcast_node<T> &b ) : my_b(b) {}
 
     void operator()(int) const {
         for (int n = 0; n < N; ++n ) {
             ASSERT( my_b.try_put( (T)n ), NULL );
         }
     }
- 
+
 };
 
 template< typename T >
-void run_parallel_broadcasts(int p, tbb::flow::broadcast_node<T>& b) {
+void run_parallel_broadcasts(tbb::flow::graph& g, int p, tbb::flow::broadcast_node<T>& b) {
     for ( int num_receivers = 1; num_receivers < R; ++num_receivers ) {
-        counting_array_receiver<T> *receivers = new counting_array_receiver<T>[num_receivers];
+        std::vector< counting_array_receiver<T> > receivers(num_receivers, counting_array_receiver<T>(g));
 
         for ( int r = 0; r < num_receivers; ++r ) {
             tbb::flow::make_edge( b, receivers[r] );
-        } 
+        }
 
         NativeParallelFor( p, native_body<T>( b ) );
 
@@ -152,13 +157,10 @@ void run_parallel_broadcasts(int p, tbb::flow::broadcast_node<T>& b) {
                 ASSERT( (int)receivers[r][n] == p, NULL );
             }
             tbb::flow::remove_edge( b, receivers[r] );
-        } 
+        }
         ASSERT( b.try_put( (T)0 ), NULL );
-        for ( int r = 0; r < num_receivers; ++r ) 
-            ASSERT( (int)receivers[r][0] == p, NULL ) ;
-
-        delete [] receivers;
-
+        for ( int r = 0; r < num_receivers; ++r )
+            ASSERT( (int)receivers[r][0] == p, NULL );
     }
 }
 
@@ -167,14 +169,13 @@ void test_parallel_broadcasts(int p) {
 
     tbb::flow::graph g;
     tbb::flow::broadcast_node<T> b(g);
-    run_parallel_broadcasts(p, b);
-    
+    run_parallel_broadcasts(g, p, b);
+
     // test copy constructor
     tbb::flow::broadcast_node<T> b_copy(b);
-    run_parallel_broadcasts(p, b_copy);
+    run_parallel_broadcasts(g, p, b_copy);
 }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
 // broadcast_node does not allow successors to try_get from it (it does not allow
 // the flow edge to switch) so we only need test the forward direction.
 template<typename T>
@@ -202,7 +203,7 @@ void test_resets() {
         if (testNo == 0) g.reset();
     }
 
-    g.reset(tbb::flow::rf_extract);
+    g.reset(tbb::flow::rf_clear_edges);
     for(T i= 0; i <= 3; i += 1) {
         b0.try_put(i);
     }
@@ -215,6 +216,7 @@ void test_resets() {
     ASSERT(!q0.try_get(j), "edge between nodes not removed");
 }
 
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
 void test_extract() {
     int dont_care;
     tbb::flow::graph g;
@@ -312,9 +314,9 @@ void test_extract() {
     }
     ASSERT(!q0.try_get(dont_care), "extra message in queue");
 }
-#endif  // TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#endif  // TBB_DEPRECATED_FLOW_NODE_EXTRACTION
 
-int TestMain() { 
+int TestMain() {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -330,9 +332,9 @@ int TestMain() {
        test_parallel_broadcasts<int_convertable_type>(p);
    }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
    test_resets<int>();
    test_resets<float>();
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
    test_extract();
 #endif
 
