@@ -1,30 +1,22 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
-    #pragma warning (push)
-    #pragma warning (disable: 4530)
-#endif
+#include "tbb/tbb_config.h"
 
-#if !(_WIN32||_WIN64) || (__MINGW64__||__MINGW32__)
+#if !(_WIN32||_WIN64) || (__MINGW64__||__MINGW32__) || __TBB_WIN8UI_SUPPORT
 
 #include "harness.h"
 
@@ -54,10 +46,6 @@ int TestMain () {
 #endif
 #include "harness.h"
 
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    #pragma warning (pop)
-#endif
-
 static int errors = 0;
 
 #define CHECK( cond ) {                                                  \
@@ -132,7 +120,7 @@ int TestMain() {
         }
 
         {
-            SAY( "Create a proxy object and call load() with good arguments but not availabe version." );
+            SAY( "Create a proxy object and call load() with good arguments but not available version." );
             char const * path[] = { ".", NULL };
             tbb::runtime_loader rtl( tbb::runtime_loader::em_status );
             SAY( "Min version too big." );
@@ -196,9 +184,9 @@ int TestMain() {
                     // Check what?
                 }
 
-                // There was a problem on Linux* OS, and still a problem on OS X*.
+                // There was a problem on Linux* OS, and still a problem on macOS*.
                 SAY( "Throw an exception." );
-                // Iterate thru all the ids first.
+                // Iterate through all the ids first.
                 for ( int id = 1; id < tbb::internal::eid_max; ++ id ) {
                     bool ex_caught = false;
                     __TBB_TRY {
